#!/usr/bin/env python

from collections import defaultdict
from errno import ENOENT
from stat import S_IFDIR, S_IFLNK, S_IFREG
from sys import argv, exit
from time import time

from fuse import FUSE, FuseOSError, Operations, LoggingMixIn

import socket
import pickle
import base64


class Client:
    """ Socket cliente para replicacao de operacoes no servidor"""
    def __init__(self, port, host):
        self.port = port
        self.host = host
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((host,int(port)))
    def send(self, data):
        self.s.send(data)
    def read(self):
        return self.s.recv(1024)

class Replicate(LoggingMixIn, Operations):
    """Filesystem replicado. Suporta apenas um nivel de directorias"""

    def __init__(self, host, port):
        self.cs = Client(port,host)        
        self.files = {}
        self.data = defaultdict(str)
        self.fd = 0
        now = time()
        self.files['/'] = dict(st_mode=(S_IFDIR | 0755), st_ctime=now,
            st_mtime=now, st_atime=now, st_nlink=2)

    def create(self, path, mode):
        self.files[path] = dict(st_mode=(S_IFREG | mode), st_nlink=1,
            st_size=0, st_ctime=time(), st_mtime=time(), st_atime=time())
        self.fd += 1
        return self.fd

    #getattr = None
    def getattr(self, path, fh=None):
        if path not in self.files:
            raise FuseOSError(ENOENT)
        st = self.files[path]
        return st  
    
    def mkdir(self, path, mode):
        self.files[path] = dict(st_mode=(S_IFDIR | mode), st_nlink=2,
                st_size=0, st_ctime=time(), st_mtime=time(), st_atime=time())
        self.files['/']['st_nlink'] += 1    
    
    def readdir(self, path, fh):
        return ['.', '..'] + [x[1:] for x in self.files if x != '/']

    def unlink(self, path):
        self.files.pop(path)

    def write(self, path, data, offset, fh):
        #self.cs.send("data:" + data)
        self.data[path] = self.data[path][:offset] + data
        self.files[path]['st_size'] = len(self.data[path])
        return len(data)

    def release(self, path, fh):
        # desenvolver version vector...
        payload = pickle.dumps(["release", [], path, self.files[path], base64.b64encode(self.data[path])], pickle.HIGHEST_PROTOCOL)
        self.cs.send(payload)
        return 0
        

    readlink = None
    removexattr = None     
    rename = None
    chmod = None
    chown = None
    setxattr = None
    statfs = None
    utimens = None
    getxattr = None 
    listxattr = None
    read = None
    truncate = None
    symlink = None
    open = None
    #def open(self, path, flags): return None
        # self.fd += 1
        # return self.fd

if __name__ == "__main__":
    if len(argv) != 4:
        print 'usage: %s <mountpoint> <host> <port>' % argv[0]
        exit(1)
    fuse = FUSE(Replicate(argv[2], argv[3]), argv[1], foreground=False)
